@@ -24,6 +24,7 @@
 #include "structure.h"
 #include "db.h"
 #include "main.h"
+#include "namedetail.h"
 
 void on_edit_cancel_clicked (GtkButton * button, gpointer user_data);
 void on_edit_save_clicked (GtkButton * button, gpointer user_data);
@@ -36,7 +37,11 @@ void tv_move_cursor (GtkTextView *textview,
 gboolean tv_focus_in (GtkWidget *widget,
                       GdkEventFocus *event,
                       gpointer user_data);
+gboolean tv_focus_out (GtkWidget *widget,
+                       GdkEventFocus *event,
+                       gpointer user_data);
 void on_unknown_year_toggled (GtkToggleButton *togglebutton, gpointer user_data);
+void on_name_clicked (GtkButton *button, gpointer user_data);
 
 static void
 add_tag (gchar *tag, GtkWidget *w, GtkWidget *pw)
@@ -50,28 +55,38 @@ add_tag (gchar *tag, GtkWidget *w, GtkWidget *pw)
 }
 
 static void
-pop_singles (GtkWidget *vbox, GSList *list, GtkWidget *pw)
+pop_singles (GtkWidget *vbox, GSList *list, GtkWidget *pw, gboolean visible)
 {
   if (list)
     {
       guint l = g_slist_length (list);
-      GtkWidget *table = gtk_table_new (l, 2, FALSE);
+      GtkWidget *table = gtk_table_new (l, gpe_get_boxspacing(), FALSE);
       guint x = 0;
-      
+    
       while (list)
         {
           GSList *next = list->next;
           edit_thing_t e = list->data;
           GtkWidget *w = gtk_entry_new ();
           GtkWidget *l;
-        
+          
           add_tag (e->tag, w, pw);
-          l = gtk_label_new (e->name);
-          gtk_misc_set_alignment(GTK_MISC(l),0.0,0.5);
+          if (strcmp(e->tag,"NAME")) /* the name field on a button */
+            {
+              l = gtk_label_new (e->name);
+              gtk_misc_set_alignment(GTK_MISC(l),1.0,0.5);
+            }
+          else
+            {
+              l = gtk_button_new_with_label(e->name);
+              g_signal_connect(G_OBJECT(l),"clicked",
+                               G_CALLBACK(on_name_clicked),
+                               gtk_widget_get_toplevel(pw));
+            }
           gtk_table_attach (GTK_TABLE (table),
                     l,
                     0, 1, x, x + 1,
-                    0, 0, 0, 0);
+                    GTK_FILL, GTK_FILL, 0, 0);
           gtk_table_attach (GTK_TABLE (table),
                     w,
                     1, 2, x, x + 1,
@@ -85,12 +100,16 @@ pop_singles (GtkWidget *vbox, GSList *list, GtkWidget *pw)
       gtk_table_set_col_spacings (GTK_TABLE (table), 2);
       gtk_container_set_border_width (GTK_CONTAINER (table), 2);
 
+      if (visible)
+        gtk_widget_show_all(table);
+      else
+        gtk_widget_hide_all(table);
       gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 4);
     }
 }
 
 static void
-build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
+build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw, gboolean visible)
 {
   GSList *child;
   GSList *singles = NULL;
@@ -106,31 +125,48 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
           {
             gchar *markup = g_strdup_printf ("<b>%s</b>", e->name);
             w = gtk_label_new (NULL);
+            if (visible)
+              gtk_widget_show(w);
+            else
+              gtk_widget_hide(w);
             gtk_label_set_markup (GTK_LABEL (w), markup);
             gtk_misc_set_alignment (GTK_MISC (w), 0, 0.5);
             g_free (markup);
-            pop_singles (vbox, singles, pw);
+            pop_singles (vbox, singles, pw, visible);
             singles = NULL;
-            gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
-            build_children (vbox, e->children, pw);
+            gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, TRUE, 0);
+            /* hidden group */
+            build_children (vbox, e->children, pw, !e->hidden);
+            if (e->hidden)
+              gtk_widget_hide(w);
           }
         break;
       
         case ITEM_MULTI_LINE:
-          pop_singles (vbox, singles, pw);
+          pop_singles (vbox, singles, pw, visible);
           singles = NULL;
           ww = gtk_text_view_new ();
+          if (visible)
+            gtk_widget_show(ww);
+          else
+            gtk_widget_hide(ww);
           gtk_text_view_set_editable (GTK_TEXT_VIEW (ww), TRUE);
           gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (ww), GTK_WRAP_WORD);
           g_signal_connect(G_OBJECT(ww),"move_cursor",
             G_CALLBACK(tv_move_cursor),NULL);
           g_signal_connect(G_OBJECT(ww),"focus-in-event",
             G_CALLBACK(tv_focus_in),NULL);
+          g_signal_connect(G_OBJECT(ww),"focus-out-event",
+            G_CALLBACK(tv_focus_out),NULL);
         
           gtk_widget_set_usize (GTK_WIDGET (ww), -1, 64);
           if (e->name)
             {
               w = gtk_frame_new (e->name);
+              if (visible)
+                gtk_widget_show(w);
+              else
+                gtk_widget_hide(w);
               gtk_container_add (GTK_CONTAINER (w), ww);
               gtk_container_set_border_width (GTK_CONTAINER (w), gpe_get_border());
             }
@@ -148,16 +184,16 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
         {
           GtkWidget *l = gtk_label_new (e->name);
           GtkWidget *hbox, *cbsched, *datecombo, *cbnoyear;
-          pop_singles (vbox, singles, pw);
+          pop_singles (vbox, singles, pw, visible);
           singles = NULL;
           
-          hbox = gtk_hbox_new (FALSE, 0);
+          hbox = gtk_hbox_new (FALSE, gpe_get_boxspacing());
           gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.5);
           gtk_box_pack_start (GTK_BOX (hbox), l, TRUE, TRUE, 0);
 
           datecombo = gtk_date_combo_new ();
           gtk_box_pack_start (GTK_BOX (hbox), datecombo, TRUE, TRUE, 0);
-
+          gtk_date_combo_clear(GTK_DATE_COMBO(datecombo));
 /*          cbsched = gtk_check_button_new_with_label (_("Schedule"));
           gtk_widget_set_sensitive (cbsched, FALSE);
           gtk_box_pack_start (GTK_BOX (hbox), cbsched, TRUE, TRUE, 0);
@@ -166,6 +202,10 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
           g_signal_connect (G_OBJECT (cbnoyear), "toggled",
 		    G_CALLBACK (on_unknown_year_toggled), datecombo);
           g_object_set_data(G_OBJECT(datecombo),"cbnoyear",cbnoyear);
+          if (visible)
+            gtk_widget_show_all(hbox);
+          else
+            gtk_widget_hide_all(hbox);
           gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, gpe_get_boxspacing());
           add_tag (e->tag, datecombo, pw);
         }
@@ -174,17 +214,18 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
         {
           GtkWidget *l = gtk_label_new (e->name);
           GtkWidget *hbox, *image, *btn;
-          pop_singles (vbox, singles, pw);
+          pop_singles (vbox, singles, pw, visible);
           singles = NULL;
           
-          hbox = gtk_hbox_new (FALSE, 0);
+          hbox = gtk_hbox_new (FALSE, gpe_get_boxspacing());
           gtk_misc_set_alignment (GTK_MISC (l), 0.0, 0.5);
-          gtk_box_pack_start (GTK_BOX (hbox), l, TRUE, TRUE, 0);
+          gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, TRUE, 0);
 
           btn = gtk_button_new();
           image = gtk_image_new();
           gtk_container_add(GTK_CONTAINER(btn),image);
           gtk_box_pack_start (GTK_BOX (hbox), btn, TRUE, TRUE, 0);
+          gtk_widget_show_all(hbox);
           gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 
             gpe_get_boxspacing());
           g_signal_connect (G_OBJECT (btn), "clicked",
@@ -198,7 +239,7 @@ build_children (GtkWidget *vbox, GSList *children, GtkWidget *pw)
       }
     }
 
-  pop_singles (vbox, singles, pw);
+  pop_singles (vbox, singles, pw, visible);
   singles = NULL;
 }
 
@@ -227,6 +268,25 @@ notebook2_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *edit_sa
   return FALSE;
 }
 
+static gboolean
+edit_window_key_press_event (GtkWidget *widget, GdkEventKey *k, GtkWidget *user_data)
+{
+  if (k->keyval == GDK_Escape) 
+  {
+    gtk_widget_destroy (GTK_WIDGET (user_data));
+    return TRUE;
+  }
+  if (k->keyval == GDK_Return) 
+  {
+    if (!g_object_get_data(G_OBJECT(widget),"inmultiline"))
+      {
+        on_edit_save_clicked (NULL, user_data);
+        return TRUE;
+      }
+  }
+  return FALSE;
+}
+
 static GtkWidget*
 create_edit (void)
 {
@@ -250,18 +310,21 @@ create_edit (void)
   else
     {
       edit = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-      action_area = gtk_hbox_new (FALSE, 0);
+      action_area = gtk_hbutton_box_new();
+	  gtk_button_box_set_spacing(GTK_BUTTON_BOX(action_area),
+		                         gpe_get_boxspacing());
+	  gtk_button_box_set_layout(GTK_BUTTON_BOX(action_area),
+		                        GTK_BUTTONBOX_END);
       vbox = gtk_vbox_new (FALSE, 0);
       gtk_box_pack_end (GTK_BOX (vbox), action_area, FALSE, FALSE, 0);
       gtk_container_add (GTK_CONTAINER (edit), vbox);
     }
-    
+  
   gtk_window_set_title (GTK_WINDOW (edit), _("Edit Contact"));
   gpe_set_window_icon (edit, "icon");
 
   notebook2 = gtk_notebook_new ();
-  gtk_box_pack_start (GTK_BOX (vbox), notebook2, 
-		      TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox), notebook2, TRUE, TRUE, 0);
 
   edit_delete = gtk_button_new_from_stock (GTK_STOCK_DELETE);
   edit_cancel = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
@@ -283,11 +346,23 @@ create_edit (void)
 		    G_CALLBACK (action_area_key_press_event), notebook2);
   g_signal_connect (G_OBJECT (notebook2), "key_press_event", 
 		    G_CALLBACK (notebook2_key_press_event), edit_save);
+  g_signal_connect (G_OBJECT (edit), "key_press_event", 
+		    G_CALLBACK (edit_window_key_press_event), edit);
 
   g_object_set_data (G_OBJECT (edit), "tooltips", tooltips);
   g_object_set_data (G_OBJECT (edit), "notebook2", notebook2);
 
-  gtk_window_set_default_size (GTK_WINDOW (edit), 240, 320);
+  if (mode_large_screen)
+    {
+        gtk_window_set_default_size (GTK_WINDOW (edit), 320, 480);
+    }
+  else
+    {
+      if (mode_landscape)
+        gtk_window_set_default_size (GTK_WINDOW (edit), 320, 240);
+      else
+        gtk_window_set_default_size (GTK_WINDOW (edit), 240, 320);
+    }
 
   return edit;
 }
@@ -299,6 +374,8 @@ edit_window (void)
   GtkWidget *book = lookup_widget (w, "notebook2");
   GSList *page;
 
+  gtk_widget_show_all(w);
+
   for (page = edit_pages; page; page = page->next)
     {
       edit_thing_t e = page->data;
@@ -308,14 +385,13 @@ edit_window (void)
 
       gtk_container_set_border_width (GTK_CONTAINER (vbox), 2);
 
-      build_children (vbox, e->children, w);
-
       gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				      GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
       gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolled_window), vbox);
-
+      gtk_widget_show_all(scrolled_window);
+      gtk_widget_show(label);
       gtk_notebook_append_page (GTK_NOTEBOOK (book), scrolled_window, label);
-      
+      build_children (vbox, e->children, w, TRUE);
       /* add categories stuff to the last page */
       if (!page->next)
         {
@@ -330,6 +406,7 @@ edit_window (void)
           g_signal_connect (G_OBJECT (catbutton), "clicked",
 		    G_CALLBACK (on_categories_clicked), w);
           g_object_set_data (G_OBJECT (w), "categories-label", catlabel);
+          gtk_widget_show_all(cathbox);
         }
     }
 
@@ -506,9 +583,112 @@ edit_person (struct person *p)
       gtk_object_set_data (GTK_OBJECT (w), "person", p);
     }
   store_special_fields (w, p);
-  gtk_widget_show_all (w);
+  gtk_widget_show (w);
 }
 
+void
+update_edit (struct person *p, GtkWidget *w)
+{
+  GtkWidget *nameentry[5];
+  int namenum = 0;
+  int i;
+  gchar *n1 = NULL, *n2 = NULL, *n3 = NULL, *n4 = NULL;
+  if (p)
+    {
+      GSList *tags = gtk_object_get_data (GTK_OBJECT (w), "tag-widgets");
+      GSList *iter;
+      for (iter = tags; iter; iter = iter->next)
+        {
+          GtkWidget *w = iter->data;
+          gchar *tag = gtk_object_get_data (GTK_OBJECT (w), "db-tag");
+          struct tag_value *v = db_find_tag (p, tag);
+          guint pos = 0;
+          if (v && v->value)
+            {
+               /* collect data for name field update */
+              if (!strcmp(v->tag,"TITLE"))
+                n1 = v->value;
+              if (!strcmp(v->tag,"GIVEN_NAME"))
+                n2 = v->value;
+              if (!strcmp(v->tag,"FAMILY_NAME"))
+                n3 = v->value;
+              if (!strcmp(v->tag,"HONORIFIC_SUFFIX"))
+                n4 = v->value;
+              if (!strcmp(v->tag,"NAME"))
+              {
+                nameentry[namenum] = w;
+                namenum++;
+              }
+
+              if (GTK_IS_EDITABLE (w))
+                {
+                  gtk_editable_delete_text(GTK_EDITABLE(w),0,-1);
+                  gtk_editable_insert_text (GTK_EDITABLE (w), v->value,
+                                            strlen (v->value), &pos);
+                }
+              else if (GTK_IS_DATE_COMBO(w))
+                {
+                  if (v->value)
+                    {
+                      guint year, month, day;
+                      GtkToggleButton *cbnoyear;
+                      
+                      sscanf (v->value, "%04d%02d%02d", &year, &month, &day);
+                      gtk_date_combo_set_date (GTK_DATE_COMBO (w), year, month, day);
+                      if (year == 0)
+                        {
+                          gtk_date_combo_ignore_year(GTK_DATE_COMBO(w),TRUE);
+                          cbnoyear = g_object_get_data(G_OBJECT(w),"cbnoyear");
+                          gtk_toggle_button_set_active(cbnoyear,TRUE);
+                        }
+                    }
+                  else
+                    gtk_date_combo_clear (GTK_DATE_COMBO (w));
+                }
+              else if (GTK_IS_IMAGE(w))
+                {
+                  g_object_set_data(G_OBJECT(w),"filename",v->value);
+                  if ((v->value) && !access(v->value,R_OK))
+                    gtk_image_set_from_file(GTK_IMAGE(w), v->value);
+                  else
+                    gtk_image_set_from_stock(GTK_IMAGE(w), 
+                      GTK_STOCK_MISSING_IMAGE, GTK_ICON_SIZE_BUTTON);
+                }
+              else  
+                gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (w)), 
+                          v->value, -1);
+            }
+        }
+    }
+  store_special_fields (w, p);
+  if (namenum)
+    {
+      gchar *ts = g_strdup(n1);
+      if (n2) 
+        {
+          n1 = g_strdup_printf("%s %s",ts,n2);
+          g_free(ts);
+          ts = n1;
+        }
+      if (n3) 
+        {
+          n1 = g_strdup_printf("%s %s",ts,n3);
+          g_free(ts);
+          ts = n1;
+        }
+      if (n4) 
+        {
+          n1 = g_strdup_printf("%s %s",ts,n4);
+          g_free(ts);
+          ts = n1;
+        }
+      for (i=0;i<namenum;i++)
+        gtk_entry_set_text(GTK_ENTRY(nameentry[i]),ts);
+      g_free(ts);
+    }
+}
+
+
 void
 on_edit_save_clicked (GtkButton * button, gpointer user_data)
 {
@@ -605,6 +785,7 @@ on_edit_bt_image_clicked (GtkWidget *bimage, gpointer user_data)
   gtk_window_set_transient_for(GTK_WINDOW(filesel),
                                GTK_WINDOW(gtk_widget_get_toplevel(bimage)));
   gtk_window_set_modal(GTK_WINDOW(filesel),TRUE);
+  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(filesel));
   
   g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filesel)->ok_button),
 		      "clicked", G_CALLBACK (store_filename), filesel);
@@ -667,6 +848,18 @@ tv_focus_in (GtkWidget *widget,
   GtkTextBuffer *buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
   gtk_text_buffer_get_start_iter(buf,&iter);
   gtk_text_buffer_place_cursor(buf,&iter);
+  g_object_set_data(G_OBJECT(gtk_widget_get_toplevel(widget)),
+                    "inmultiline",(void*)1);  
+  return FALSE;
+}
+
+gboolean 
+tv_focus_out (GtkWidget *widget,
+             GdkEventFocus *event,
+             gpointer user_data)
+{
+  g_object_set_data(G_OBJECT(gtk_widget_get_toplevel(widget)),
+                    "inmultiline",(void*)0);  
   return FALSE;
 }
 
@@ -677,3 +870,13 @@ on_unknown_year_toggled (GtkToggleButton *togglebutton, gpointer user_data)
   
   gtk_date_combo_ignore_year(cb, gtk_toggle_button_get_active(togglebutton));
 }
+
+void
+on_name_clicked (GtkButton *button, gpointer user_data)
+{
+  GtkWindow *edit = user_data;
+  struct person *p;
+
+  p = g_object_get_data (G_OBJECT (edit), "person");
+  do_edit_name_detail(edit, p);
+}
